;;; Avoid a conflict with the package :GL of SBCL's CLX that was introduced
;;; with the GLX extensions support of SBCL CLX version 0.6.1.
(delete-package 'gl)
(require :cube-modeller)
(in-package "SB-ALIEN")

(let ((function (symbol-function 'try-reopen-shared-object)))
  (setf (symbol-function 'try-reopen-shared-object)
	#'(lambda (obj)
	    (declare (type shared-object obj))

	    ;; (format  *trace-output* "~&>>> 1) desired shared-object: ~d" (shared-object-file obj))
            (setf (shared-object-file obj)
                  (file-namestring (shared-object-file obj)))
	    ;; (format  *trace-output* "~&>>> 2) now searching instead: ~d" (shared-object-file obj))
            
            (funcall function obj))))

(in-package :cl-user)

(defun configure-freetype ()
  (setf mcclim-freetype:*freetype-font-path*
        (merge-pathnames (make-pathname :directory '(:relative "fonts"))
                         (make-pathname :directory (pathname-directory sb-ext:*core-pathname*))))
  
  (flet ((set-font (family face filename)
           (setf (cdr (assoc (list family face) mcclim-freetype::*families/faces*
                             :test #'equal))
                 filename)))
    (set-font :fix :roman "Andale_Mono.ttf")
    (set-font :fix :bold "Andale_Mono.ttf")))

(defun get-command-line-argument (argument-name)
  (let ((position (position argument-name sb-ext:*posix-argv* :test #'string-equal)))
    (when (and position
               (<= (+ 2 position) ;; there is an argument after its name
                   (length sb-ext:*posix-argv*)))
      (nth (1+ position) sb-ext:*posix-argv*))))
  
(setf *invoke-debugger-hook* nil) ;; Use SBCL's built-in debugger while loading the foreign objects.

(sb-ext:save-lisp-and-die "cube-modeller"
                          :executable t
			  :toplevel (lambda ()
                                      (setf *invoke-debugger-hook* #'clim-debugger:debugger)

                                      (format t "~&~%=============== LED Cube Modeller ===============~&~%")

                                      (when (position "--help" sb-ext:*posix-argv* :test #'string-equal)
                                        (format t "
Usage: cube-modeller [--help] [--host <host-name or ip>] [-port <port>]
                              [--cube-tty <device>] [--frame-delay <delay>]

  --help          show this screen

  --host          host-name or ip of the cubeserver
  --host-name
  --hostname
  --ip

  --port          port of the cubeserver

  --cube-tty      path to the serial device of the cube
  --cubetty

  --frame-delay   set delay between frames (in milliseconds)

")
                                        (sb-ext:quit))
                                      
				      (setf cube::*device-file*
                                            (or (get-command-line-argument "--cubetty")
                                                (get-command-line-argument "--cube-tty")))
                                      (when cube::*device-file*
					(format t "~&Writing cube data to device file \"~d\".~&~%" cube::*device-file*))


                                      (setf cube-modeller::*animation-pathname*
                                            (merge-pathnames (make-pathname :directory '(:relative "data"))
                                                             (make-pathname :directory (pathname-directory sb-ext:*core-pathname*))))
                                      
                                      (configure-freetype)

                                      (cube:set-server-address (or (get-command-line-argument "--host")
                                                                   (get-command-line-argument "--hostname")
                                                                   (get-command-line-argument "--host-name")
                                                                   (get-command-line-argument "--ip"))
                                                               (parse-integer (get-command-line-argument "--port")
                                                                              :junk-allowed t))

                                      (let ((frame-delay (or (get-command-line-argument "--frame-delay")
                                                             (get-command-line-argument "--framedelay"))))
                                        (setf frame-delay (ignore-errors (/ (coerce (parse-integer frame-delay :junk-allowed t)
                                                                                    'float)
                                                                            1000)))
                                        (when frame-delay
                                          (format t "~&Setting frame delay to ~a.~&~%" frame-delay)
                                          (setf LED-cube:*default-frame-delay* frame-delay)))
                                      
				      (cube-modeller:start)
				      (sb-ext:quit)))
