(in-package :cube-modeller)

(defstruct fakecube
  origin
  size)

(defstruct size
  x y z)

(defstruct origin
  x y z)

(defun copy-and-update-in-direction (source-list direction &key new-value)
  (let ((list (copy-list source-list)))
    (setf (nth direction list)
          new-value)
    list))

;; (defvar *number-of-displayed-leds* 125)
;; (defvar *led-count* 0)
(defun draw-leds (cube viewray &key draw-led recursion)
  "Draw leds from back to front with the BSP algorithm (binary space partitioning / painters algorithm).
  This is a very simplified version of the BSP algorithm that only works
  because of the simplicity and regularity of our scene."
  (declare (ignore recursion))
;;   (unless recursion
;;     (setf *led-count* 0))
  (if (equal (fakecube-size cube)
             '(1 1 1))
      ;; end of recursion
      (funcall draw-led (fakecube-origin cube))
;;       (progn
;;         (incf *led-count*)
;;         (when (<= *led-count* *number-of-displayed-leds*)
;;           (funcall draw-led (fakecube-origin cube))))
      
      ;; recurse
      (let* ((origin (fakecube-origin cube))
             (size (fakecube-size cube))

             (direction-dimension (apply #'max size))
             (direction (position direction-dimension size))
               
             (back-direction-dimension (floor (/ direction-dimension
                                                  2)))
             (back-cube (make-fakecube :origin origin
                                       :size (copy-and-update-in-direction
                                              size direction
                                              :new-value back-direction-dimension)))
             (front-cube (make-fakecube :origin (copy-and-update-in-direction
                                                origin direction
                                                :new-value (+ (nth direction origin)
                                                              back-direction-dimension))
                                        :size (copy-and-update-in-direction
                                               size direction
                                               :new-value (- (nth direction size)
                                                             back-direction-dimension)))))
        ;; (list front-cube back-cube))))
        (if (< (nth direction viewray)
               0.0)
            (progn
              (draw-leds back-cube viewray :draw-led draw-led :recursion t)
              (draw-leds front-cube viewray :draw-led draw-led :recursion t))
            (progn
              (draw-leds front-cube viewray :draw-led draw-led :recursion t)
              (draw-leds back-cube viewray :draw-led draw-led :recursion t)))))

  nil)

(defun draw-led (position)
  (gl:push-matrix)
  (destructuring-bind (x y z)
      position
    (gl:translate-f (- (* y *led-distance*)) (* z *led-distance*) (- (* x *led-distance*)))
    (let* ((LED-intensity-ASCII (aref (current-cube) x y z))
           (LED-intensity (- (char-code LED-intensity-ASCII) 48))
           (grey-value  (- .9f0 (* (- 5 LED-intensity)
                                   (/ 0.8 5))))
           (alpha-value  (- .9f0 (* (- 5 LED-intensity)
                                    (/ 0.5 5))))
           (old-grey-value (* (- 5 LED-intensity)
                              (/ 0.9 5))))
      (if (eq LED-intensity 0)
          (sgum:with-single-float-values (emission 0f0 0f0 0f0 0f0)
            (gl:material-fv gl:+front+ gl:+emission+ emission))
          (sgum:with-single-float-values (emission .1f0 0f0 0f0 0f0)
            (gl:material-fv gl:+front+ gl:+emission+ emission)))
;;     (sgum:with-single-float-values (diffuse 0.9f0 old-grey-value old-grey-value 1f0)
      (sgum:with-single-float-values (diffuse grey-value 0f0 0f0 alpha-value)
        (gl:material-fv gl:+front+ gl:+diffuse+ diffuse)))

    (gl:load-name (+ (* (+ (* 5 z) y) 5) x)) ;; LED id
;;     (gl:enable gl:+depth-test+)
      (draw-sphere *LED-DIAMETER*)
;;     (gl:disable gl:+depth-test+)
    (gl:end) ;; of gl:load-name
    (gl:pop-matrix)))
